@@ -8,6 +8,7 @@
 
 #include <stdio.h>
 
+#include "config.h"
 #ifdef FLOAT_TYPE
 #include <math.h>
 #endif
@@ -27,8 +28,12 @@ void (*SLang_Exit_Function)(char *) = NULL;
 int SLang_Trace = 0;
 char SLang_Trace_Function[32];
 
-
+#ifdef MSWINDOWS
+SLang_Name_Type *SLang_Name_Table;
+#else
 SLang_Name_Type SLang_Name_Table[SLANG_MAX_SYMBOLS];
+#endif
+
 static int SLang_Name_Table_Ofs[256];
 SLName_Table *SLName_Table_Root;
 
@@ -50,11 +55,15 @@ static SLBlock_Type Lang_Interp_Stack_Static[SLANG_MAX_TOP_STACK];
 static SLBlock_Type *Lang_Interp_Stack_Ptr = Lang_Interp_Stack_Static;
 static SLBlock_Type *Lang_Interp_Stack = Lang_Interp_Stack_Static;
 
+#ifdef MSWINDOWS
+SLang_Object_Type *SLRun_Stack;
+SLang_Object_Type *SLStack_Pointer;
+static SLang_Object_Type *SLStack_Pointer_Max;
+#else
 SLang_Object_Type SLRun_Stack[SLANG_MAX_STACK_LEN];
 SLang_Object_Type *SLStack_Pointer = SLRun_Stack;
-
 static SLang_Object_Type *SLStack_Pointer_Max = SLRun_Stack + SLANG_MAX_STACK_LEN;
-
+#endif
 
 /* Might want to increase this. */
 #define MAX_LOCAL_STACK 200
@@ -956,7 +965,12 @@ static void lang_do_else(unsigned char type, SLBlock_Type *addr, SLBlock_Type *a
 
 static void lang_dump(char *s)
 {
-   fputs(s, stderr);
+   while (*s)
+     {
+	if (*s == '\n') fputc ('\r', stderr);
+	fputc (*s, stderr);
+	s++;
+     }
 }
 
 void (*SLang_Dump_Routine)(char *) = lang_dump;
@@ -1766,6 +1780,7 @@ void SLadd_name(char *name, long addr, unsigned char main_type, unsigned char su
 	if (main_type != mtype)
 	  {
 	     SLang_Error = DUPLICATE_DEFINITION;
+	     SLang_doerror (name);
 	     return;
 	  }
      }   
@@ -2317,6 +2332,7 @@ static int interp_variable_eqs(char *name)
        && (mtype != SLANG_IVARIABLE))
      {
 	SLang_Error = DUPLICATE_DEFINITION;
+	SLang_doerror (name);
 	return(1);
      }
 
@@ -2361,7 +2377,12 @@ static int interp_variable_eqs(char *name)
 }
 
    
-
+#ifdef NO_STDLIB_H
+/* Oh dear.  Where is the prototype for atof?  If not in stdlib, then 
+ * I do not know where.
+ */
+extern double atof ();
+#endif
 
 /* a literal */
 static int interp_push_number(char *t)
@@ -2373,7 +2394,7 @@ static int interp_push_number(char *t)
    FLOAT x = 0.0;
 #endif
 
-   stype = slang_guess_type(t);
+   stype = SLang_guess_type(t);
    if (stype == STRING_TYPE) return(0);
    if (stype == INT_TYPE)
      {
@@ -2985,7 +3006,19 @@ int SLang_add_table(SLang_Name_Type *table, char *table_name)
    
    if (init == 0)
      {
+#ifdef MSWINDOWS	
+	SLRun_Stack = (SLang_Object_Type *) SLCALLOC (SLANG_MAX_STACK_LEN,
+						      sizeof (SLang_Object_Type));
+	if (SLRun_Stack == NULL)
+	  {
+	     return 0;
+	  }
+	SLStack_Pointer = SLRun_Stack;
+	SLStack_Pointer_Max = SLRun_Stack + SLANG_MAX_STACK_LEN;
+#endif
+	
 	init = 1;
+	
 	for (i = 1; i < 256; i++) SLang_Name_Table_Ofs[i] = -1;
 	SLang_Name_Table_Ofs[0] = 0;
 	
