@@ -6,8 +6,8 @@
  * You may distribute under the terms of either the GNU General Public
  * License or the Perl Artistic License.
  */
-#define SLANG_VERSION 9915
-#if defined(__WATCOMC__)
+#define SLANG_VERSION 9922
+#if defined(__WATCOMC__) && !defined(__QNX__)
 #include <slconfig.h>
 #endif
 
@@ -31,7 +31,7 @@ extern "C" {
 #undef VOID
 #endif
 
-#if defined(msdos) && !defined(DOS386)
+#if defined(msdos) && !defined(DOS386) & !defined(__WIN32__)
 #  ifdef __SC__
 #    include <dos.h>
 #  endif
@@ -323,6 +323,7 @@ SLuser_Object_Type;
    extern int SLang_add_unary_op (unsigned char, VOID *);
    extern int SLang_add_binary_op (unsigned char, unsigned char, VOID *);
    extern int SLang_register_class (unsigned char, VOID *, VOID *);
+   extern int SLang_add_copy_operation (unsigned char, VOID *);
 
    extern long *SLang_pop_pointer(unsigned char *, unsigned char *, int *);
    /* Returns a pointer to object of type *p1,*p2 on top of stack. 
@@ -342,10 +343,6 @@ SLuser_Object_Type;
    /* The normal SLang_push_string mallocs space for the string.  This one
       does not.  DO NOT FREE IT IF YOU USE THIS ROUTINE */
 
-   extern int SLdefine_for_ifdef (char *);
-   /* Adds a string to the SLang #ifdef preparsing defines. SLang already
-      defines MSDOS, UNIX, and VMS on the appropriate system. */
-      
    extern int SLang_is_defined(char *);
    /* Return non-zero is p1 is defined otherwise returns 0. */
    
@@ -426,12 +423,14 @@ extern char *SLmake_nstring (char *, int);
  * string.
  */
 
+   extern int SLang_guess_type (char *);
 /* SLang tty interface */
 extern int SLtt_Baud_Rate;
 extern unsigned long SLtt_Num_Chars_Output;
    
-#ifndef VMS
+#ifdef unix
 extern int SLang_TT_Baud_Rate;
+extern int SLang_TT_Read_FD;
 #endif
    
 extern int SLang_init_tty (int, int, int);
@@ -449,7 +448,12 @@ extern int SLang_init_tty (int, int, int);
 
 extern void SLang_reset_tty (void);
 /* Resets tty to what it was prior to a call to SLang_init_tty */
-
+#ifdef unix
+extern void SLtty_set_suspend_state (int);
+   /* If non-zero argument, terminal driver will be told to react to the
+    * suspend character.  If 0, it will not.
+    */
+#endif
 extern unsigned int SLang_getkey (void);
 /* reads a single key from the tty.  If the read fails, 0xFFFF is returned. */
 extern void SLang_ungetkey_string (unsigned char *, int);
@@ -631,6 +635,7 @@ extern int SLtt_Term_Cannot_Insert;
 extern int SLtt_Term_Cannot_Scroll;
 extern int SLtt_Use_Ansi_Colors;
 extern int SLtt_Ignore_Beep;
+extern int SLtt_Try_Termcap;
 extern int SLtt_Has_Alt_Charset;
    
 #if defined(VMS) || defined(unix)
@@ -666,18 +671,48 @@ extern void SLtt_init_video (void);
 extern void SLtt_reset_video (void);
 extern void SLtt_get_terminfo(void);
 extern int SLtt_set_cursor_visibility (int);
-#if defined(VMS) || defined(unix) 
+#if defined(VMS) || defined(unix)
+# ifndef __GO32__
 extern void SLtt_enable_cursor_keys(void);
 extern void SLtt_set_term_vtxxx(int *);
 extern void SLtt_set_color_esc (int, char *);
 extern void SLtt_wide_width(void);
 extern void SLtt_narrow_width(void);
+extern int SLtt_set_mouse_mode (int, int);
+#  ifdef unix
+   extern char *SLtt_tgetstr (char *);
+   extern int SLtt_tgetnum (char *);
+   extern int SLtt_tgetflag (char *);
+#  endif
+# endif
 #endif
 extern void SLtt_set_color_object (int, SLtt_Char_Type);
 extern void SLtt_set_color (int, char *, char *, char *);
 extern void SLtt_set_mono (int, char *, SLtt_Char_Type);
 extern void SLtt_add_color_attribute (int, SLtt_Char_Type);
      
+/*-------------------------------------------------------------------------
+ * Preprocessor
+ */
+typedef struct 
+{
+   int this_level;
+   int exec_level;
+   int prev_exec_level;
+   char preprocess_char;
+   char comment_char;
+}
+SLPreprocess_Type;
+
+extern int SLprep_open_prep (SLPreprocess_Type *);
+extern void SLprep_close_prep (SLPreprocess_Type *);
+extern int SLprep_line_ok (char *, SLPreprocess_Type *);
+   extern int SLdefine_for_ifdef (char *);
+   /* Adds a string to the SLang #ifdef preparsing defines. SLang already
+      defines MSDOS, UNIX, and VMS on the appropriate system. */
+      
+   
+   
 /* ----------------------------------------------------------------------- */
 /* Screen management functions */
 
@@ -685,7 +720,10 @@ extern void SLtt_add_color_attribute (int, SLtt_Char_Type);
 extern void SLsmg_fill_region (int, int, int, int, unsigned char);
 #ifndef pc_system
 extern void SLsmg_set_char_set (int);
+extern int SLsmg_Scroll_Hash_Border;
 #endif
+extern void SLsmg_suspend_smg (void);
+extern void SLsmg_resume_smg (void);
 extern void SLsmg_erase_eol (void);
 extern void SLsmg_gotorc (int, int);
 extern void SLsmg_erase_eos (void);
@@ -710,7 +748,8 @@ extern void SLsmg_draw_hline (int);
 extern void SLsmg_draw_vline (int);
 extern void SLsmg_draw_object (int, int, unsigned char);
 extern void SLsmg_draw_box (int, int, int, int);
-
+extern int SLsmg_get_column(void);
+extern int SLsmg_get_row(void);
 extern int SLsmg_Display_Eight_Bit;
 extern int SLsmg_Tab_Width;
 
@@ -731,6 +770,10 @@ extern int SLsmg_Tab_Width;
 # define SLSMG_LLCORN_CHAR	'm'
 # define SLSMG_LRCORN_CHAR	'j'
 # define SLSMG_CKBRD_CHAR	'a'
+# define SLSMG_RTEE_CHAR	'u'
+# define SLSMG_LTEE_CHAR	't'
+# define SLSMG_UTEE_CHAR	'w'
+# define SLSMG_DTEE_CHAR	'v'
 #endif
 /* ----------------------------------------------------------------------- */
 
@@ -885,6 +928,7 @@ typedef struct
    int must_match_bol;		       /* true if it must match beginning of line */
    unsigned char must_match_str[16];   /* 15 char null term substring */
    int osearch;			       /* 1 if ordinary search suffices */
+   int min_length;		       /* minimum length the match must be */
    int beg_matches[10];		       /* offset of start of \( */
    int end_matches[10];		       /* length of nth submatch
 					* Note that the entire match corresponds
