@@ -8,6 +8,8 @@
 #include <string.h>
 #include <time.h>
 
+#include "config.h"
+
 #include "slang.h"
 #include "_slang.h"
 
@@ -61,9 +63,7 @@ static int Automatic_Margins;
 /* static int No_Move_In_Standout; */
 static int Worthless_Highlight;
 
-#ifdef linux
 static int Linux_Console;
-#endif
 
 /* It is crucial that JMAX_COLORS must be less than 128 since the high bit
  * is used to indicate a character from the ACS (alt char set).  The exception
@@ -100,8 +100,10 @@ static Ansi_Color_Type Ansi_Color_Map[JMAX_COLORS] =
    {RGB(1, 1, 0, 1, 1, 1), SLTT_REV_MASK, NULL},
    {RGB(1, 0, 1, 1, 1, 1), SLTT_REV_MASK, NULL},
    {RGB(0, 0, 0, 0, 1, 1), SLTT_REV_MASK, NULL},
-   {RGB(0, 0, 0, 1, 1, 0), SLTT_REV_MASK, NULL},
-   {RGB(0, 0, 0, 1, 0, 1), SLTT_REV_MASK, NULL}
+   {RGB(0, 1, 0, 1, 1, 1), SLTT_REV_MASK, NULL},
+   {RGB(0, 1, 0, 1, 1, 1), SLTT_REV_MASK, NULL},
+   {RGB(0, 1, 0, 1, 1, 1), SLTT_REV_MASK, NULL},
+   {RGB(0, 1, 0, 1, 1, 1), SLTT_REV_MASK, NULL}
 };
 
 
@@ -755,6 +757,8 @@ void SLtt_set_alt_char_set (int i)
    if (SLtt_Has_Alt_Charset == 0) return;
    if (i == last_i) return;
    SLtt_write_string (i ? Start_Alt_Chars_Str : End_Alt_Chars_Str );
+   /* if (i) Current_Fgbg |= SLTT_ALTC_MASK; 
+    else Current_Fgbg &= ~SLTT_ALTC_MASK; */
    last_i = i;
 }
 
@@ -762,6 +766,7 @@ static void write_attributes (SLtt_Char_Type fgbg)
 {
    int bg0, fg0;
    
+   if (Worthless_Highlight) return;
    if (fgbg == Current_Fgbg) return;
    
    /* Before spitting out colors, fix attributes */
@@ -770,6 +775,10 @@ static void write_attributes (SLtt_Char_Type fgbg)
 	if (Current_Fgbg & ATTR_MASK)
 	  {
 	     SLtt_write_string(Norm_Vid_Str);
+	     /* In case normal video turns off ALL attributes: */
+	     if (fgbg & SLTT_ALTC_MASK) 
+	       Current_Fgbg &= ~SLTT_ALTC_MASK;
+	     SLtt_set_alt_char_set (0);
 	  }
 	
 	if ((fgbg & SLTT_ALTC_MASK) 
@@ -918,9 +927,15 @@ static void send_attr_str(unsigned short *s)
 			    Cursor_c += (int) (p - out);
 			    p = out;
 			 }
+
 		       if (SLtt_Use_Ansi_Colors && (NULL != Ansi_Color_Map[color & 0x7F].custom_esc))
 			 {
 			    SLtt_write_string (Ansi_Color_Map[color & 0x7F].custom_esc);
+			    /* Just in case the custom escape sequence screwed up
+			     * the alt character set state...
+			     */
+	                    if ((attr & SLTT_ALTC_MASK) != (Current_Fgbg & SLTT_ALTC_MASK))
+			      SLtt_set_alt_char_set ((int) (attr & SLTT_ALTC_MASK));
 			    Current_Fgbg = attr;
 			 }
 		       else write_attributes (attr);
@@ -1095,8 +1110,9 @@ void SLtt_smart_puts(unsigned short *neww, unsigned short *oldd, int len, int ro
 #ifdef unix
 #define EXTERN extern
 
-#ifdef SLANG_TERMINFO
+#ifndef USE_TERMCAP
 static char *Tbuf;
+static char *Tstr_Buf;
 
 #define tgetstr SLtt_tigetstr
 #define tgetent SLtt_tigetent
@@ -1108,7 +1124,7 @@ EXTERN char *tgetstr(char *, char **);
 EXTERN int tgetent(char *, char *);
 EXTERN int tgetnum(char *);
 EXTERN int tgetflag(char *);
-static char Tstr_Buf[512];
+static char Tstr_Buf[1024];
 static char Tbuf[4096];
 #define TGETNUM tgetnum
 #define TGETFLAG tgetflag
@@ -1144,13 +1160,31 @@ static char *my_tgetstr(char *what, char **p)
    return(what);
 }
 
-static int Vt100_Like = 0;
+char *SLtt_tgetstr (char *s)
+{
+#ifdef USE_TERMCAP
+   static
+#endif
+     char *p = Tstr_Buf;
+   return my_tgetstr (s, &p);
+}
 
+int SLtt_tgetnum (char *s)
+{
+   return TGETNUM (s);
+}
+int SLtt_tgetflag (char *s)
+{
+   return TGETFLAG (s);
+}
+
+
+static int Vt100_Like = 0;
 
 void SLtt_get_terminfo (void)
 {
    char *term, *t, ch;
-   char *p;
+   int is_xterm;
    
    SLtt_Use_Ansi_Colors = (NULL != getenv ("COLORTERM"));
 
@@ -1159,116 +1193,178 @@ void SLtt_get_terminfo (void)
 	SLang_exit_error("TERM environment variable needs set.");
      }
    
+   Linux_Console = (!strncmp (term, "linux", 5)
 #ifdef linux
-   Linux_Console = !strncmp(term, "con", 3);
+		    || !strncmp(term, "con", 3)
 #endif
+		    );
       
    t = term;
    
    if (strcmp(t, "vt52") && (*t++ == 'v') && (*t++ == 't')
        && (ch = *t, (ch >= '1') && (ch <= '9'))) Vt100_Like = 1; 
 
-#ifdef SLANG_TERMINFO
+#ifndef USE_TERMCAP
    if (NULL == (Tbuf = tgetent (term))) 
      {
-	char err_buf[256];
+	char err_buf[512];
 	/* Special cases. */
 	if (Vt100_Like
-#ifdef linux
 	    || Linux_Console
-#endif
 	    || !strcmp (term, "screen"))
 	  {
 	     int vt102 = 1;
 	     if (!strcmp (term, "vt100")) vt102 = 0;
 	     SLtt_Screen_Cols = 80;
 	     SLtt_Screen_Rows = 24;
-#ifdef linux
 	     if (Linux_Console) SLtt_Screen_Rows++;
-#endif
    	     SLtt_set_term_vtxxx (&vt102);
 	     return;
 	  }
 	sprintf (err_buf, "Unknown terminal: %s\n\
 Check the TERM environment variable.\n\
-Also make sure that the terminal is defined in the terminfo database.", term);
+Also make sure that the terminal is defined in the terminfo database.\n\
+Alternatively, set the TERMCAP environment variable to the desired\n\
+termcap entry.", term);
 	SLang_exit_error(err_buf);
      }
-   p = Tbuf;
-#else
+   Tstr_Buf = Tbuf;
+#else				       /* USE_TERMCAP */
    if (1 != tgetent(Tbuf, term)) SLang_exit_error("Unknown terminal.");
-   p = Tstr_Buf;
-#endif
+#endif				       /* NOT USE_TERMCAP */
    
-   if ((NULL == (Cls_Str = my_tgetstr("cl", &p))) 
-       || (NULL == (Curs_Pos_Str = my_tgetstr("cm", &p))))
+   if ((NULL == (Cls_Str = SLtt_tgetstr("cl"))) 
+       || (NULL == (Curs_Pos_Str = SLtt_tgetstr("cm"))))
      {
 	SLang_exit_error("Terminal not powerful enough for SLang.");
      }
    
-   if ((NULL == (Ins_Mode_Str = my_tgetstr("im", &p)))
-       || ( NULL == (Eins_Mode_Str = my_tgetstr("ei", &p)))
-       || ( NULL == (Del_Char_Str = my_tgetstr("dc", &p))))
+   if ((NULL == (Ins_Mode_Str = SLtt_tgetstr("im")))
+       || ( NULL == (Eins_Mode_Str = SLtt_tgetstr("ei")))
+       || ( NULL == (Del_Char_Str = SLtt_tgetstr("dc"))))
      SLtt_Term_Cannot_Insert = 1;
    
-   Visible_Bell_Str = my_tgetstr ("vb", &p);
-   Curs_Up_Str = my_tgetstr ("up", &p);
-   Rev_Scroll_Str = my_tgetstr("sr", &p);
-   Del_N_Lines_Str = my_tgetstr("DL", &p);
-   Add_N_Lines_Str = my_tgetstr("AL", &p);
+   Visible_Bell_Str = SLtt_tgetstr ("vb");
+   Curs_Up_Str = SLtt_tgetstr ("up");
+   Rev_Scroll_Str = SLtt_tgetstr("sr");
+   Del_N_Lines_Str = SLtt_tgetstr("DL");
+   Add_N_Lines_Str = SLtt_tgetstr("AL");
    
    /* Actually these are used to initialize terminals that use cursor 
     * addressing.  Hard to believe.
     */
-   Term_Init_Str = my_tgetstr ("ti", &p);
-   Term_Reset_Str = my_tgetstr ("te", &p);
+   Term_Init_Str = SLtt_tgetstr ("ti");
+   Term_Reset_Str = SLtt_tgetstr ("te");
    
    /* Make up for defective termcap/terminfo databases */
    if ((Vt100_Like && (term[2] != '1'))
-#ifdef linux
        || Linux_Console
-#endif
        )
      {
 	if (Del_N_Lines_Str == NULL) Del_N_Lines_Str = "\033[%dM";
 	if (Add_N_Lines_Str == NULL) Add_N_Lines_Str = "\033[%dL";
      }
    
-   Scroll_R_Str = my_tgetstr("cs", &p);
+   Scroll_R_Str = SLtt_tgetstr("cs");
    
    if ((SLtt_Screen_Rows == 0) || (SLtt_Screen_Cols == 0))
      {
+	char *envptr;
 	if ((SLtt_Screen_Cols = TGETNUM("co")) <= 0) SLtt_Screen_Cols = 80;
 	if ((SLtt_Screen_Rows = TGETNUM("li")) <= 0) SLtt_Screen_Rows = 24;
+	envptr = getenv ("LINES");
+	if (envptr != NULL)
+	  {
+	     if ((SLtt_Screen_Rows = atoi (envptr)) <= 0) 
+	       SLtt_Screen_Rows = 24;
+	  }
+	
+	envptr = getenv ("COLUMNS");
+	if (envptr != NULL)
+	  {
+	     if ((SLtt_Screen_Cols = atoi (envptr)) <= 0) 
+	       SLtt_Screen_Cols = 80;
+	  }
      }
    
+   is_xterm = !strncmp (term, "xterm", 5);
    
    if ((Scroll_R_Str == NULL) 
        || (((NULL == Del_N_Lines_Str) || (NULL == Add_N_Lines_Str))
 	   && (NULL == Rev_Scroll_Str)))
-     SLtt_Term_Cannot_Scroll = 1;
+     {
+	if (is_xterm)
+	  {
+	     /* Defective termcap mode!!!! */
+	     SLtt_set_term_vtxxx (&is_xterm);
+	  }
+	else SLtt_Term_Cannot_Scroll = 1;
+     }
    
-   Del_Eol_Str = my_tgetstr("ce", &p);
+   Del_Eol_Str = SLtt_tgetstr("ce");
 
-   Rev_Vid_Str = my_tgetstr("mr", &p);
-   if (Rev_Vid_Str == NULL) Rev_Vid_Str = my_tgetstr("so", &p);
+   Rev_Vid_Str = SLtt_tgetstr("mr");
+   if (Rev_Vid_Str == NULL) Rev_Vid_Str = SLtt_tgetstr("so");
    
-   Bold_Vid_Str = my_tgetstr("md", &p);
-   Blink_Vid_Str = my_tgetstr("mb", &p);
-   UnderLine_Vid_Str = my_tgetstr("us", &p);
+   Bold_Vid_Str = SLtt_tgetstr("md");
+   Blink_Vid_Str = SLtt_tgetstr("mb");
+   UnderLine_Vid_Str = SLtt_tgetstr("us");
    
-   Start_Alt_Chars_Str = my_tgetstr ("as", &p);
-   End_Alt_Chars_Str = my_tgetstr ("ae", &p);
-   Enable_Alt_Char_Set = my_tgetstr ("eA", &p);
+   Start_Alt_Chars_Str = SLtt_tgetstr ("as");   /* smacs */
+   End_Alt_Chars_Str = SLtt_tgetstr ("ae");   /* rmacs */
+   Enable_Alt_Char_Set = SLtt_tgetstr ("eA");   /* enacs */
+   SLtt_Graphics_Char_Pairs = SLtt_tgetstr ("ac");
    
-   if ((NULL == (SLtt_Graphics_Char_Pairs = my_tgetstr ("ac", &p)))
-       && Vt100_Like)
+   /* I have had it with defective console/linux terminfo entries provided 
+    * by ncurses.  They are not reliable.  I cannot track every ncurses 
+    * release and fudge my stuff to work with defective stuff.  For that
+    * reason, I am hardcoding some things.
+    */
+   if (Linux_Console)
+     {
+#if 0
+	char *lgcp = "l\332m\300k\277j\331u\264t\303v\301w\302q\304x\263n\053o\176s\137`\004a\260f\370g\361~\011,\020+\021.\031-\030h\261i\0250\333";
+
+	if ((SLtt_Graphics_Char_Pairs == NULL)
+	    || (0 == strncmp (SLtt_Graphics_Char_Pairs, lgcp, 8)))
+	  {
+	     Start_Alt_Chars_Str = "\033(B\033)U\016";
+	     End_Alt_Chars_Str = "\033(B\033)0\017";
+	     Enable_Alt_Char_Set = NULL;
+	  }
+#else
+	char *lgcp = "`\004a\261f\370g\361h\260j\331k\277l\332m\300n\305o\302q\304r\362s_t\303u\264v\301w\302x\263y\371z\372{\373|\374}\375~";
+	
+	if ((SLtt_Graphics_Char_Pairs == NULL)
+	    || (0 == strncmp (SLtt_Graphics_Char_Pairs, lgcp, 16)))
+	  {
+	     SLtt_Graphics_Char_Pairs = lgcp;
+	     Start_Alt_Chars_Str = "\033[11m";
+	     End_Alt_Chars_Str = "\033[10m";
+	     Enable_Alt_Char_Set = NULL;
+	  }
+#endif
+     }
+   else if (NULL == SLtt_Graphics_Char_Pairs)
+     {
+	/* make up for defective termcap/terminfo */
+	if (Vt100_Like)
+	  {
+	     Start_Alt_Chars_Str = "\016";
+	     End_Alt_Chars_Str = "\017";
+	     Enable_Alt_Char_Set = "\033)0";
+	  }
+     }
+   
+   if (is_xterm)
      {
 	Start_Alt_Chars_Str = "\016";
 	End_Alt_Chars_Str = "\017";
+	Enable_Alt_Char_Set = "\033(B\033)0";
      }
    
-   if ((Start_Alt_Chars_Str == NULL) || (End_Alt_Chars_Str == NULL))
+   if ((SLtt_Graphics_Char_Pairs == NULL) && 
+       ((Start_Alt_Chars_Str == NULL) || (End_Alt_Chars_Str == NULL)))
      {
 	SLtt_Has_Alt_Charset = 0;
 	Enable_Alt_Char_Set = NULL;
@@ -1277,15 +1373,15 @@ Also make sure that the terminal is defined in the terminfo database.", term);
    
        
    
-   if (NULL == (Norm_Vid_Str = my_tgetstr("me", &p))) 
+   if (NULL == (Norm_Vid_Str = SLtt_tgetstr("me"))) 
      {
-	Norm_Vid_Str = my_tgetstr("se", &p);
+	Norm_Vid_Str = SLtt_tgetstr("se");
      }
    
-   Cursor_Invisible_Str = my_tgetstr("vi", &p);
-   Cursor_Visible_Str = my_tgetstr("vs", &p);
+   Cursor_Invisible_Str = SLtt_tgetstr("vi");
+   Cursor_Visible_Str = SLtt_tgetstr("vs");
    
-   Curs_F_Str = my_tgetstr("RI", &p);
+   Curs_F_Str = SLtt_tgetstr("RI");
    
 #if 0
    if (NULL != Curs_F_Str)
@@ -1298,6 +1394,11 @@ Also make sure that the terminal is defined in the terminfo database.", term);
    Automatic_Margins = TGETFLAG ("am");
    /* No_Move_In_Standout = !TGETFLAG ("ms"); */
    Worthless_Highlight = TGETFLAG ("xs");
+   
+   if (Worthless_Highlight == 0)
+     {
+	Worthless_Highlight = (TGETNUM ("sg") > 0);
+     }
 }
 
 #endif
@@ -1317,6 +1418,11 @@ void SLtt_get_terminfo ()
 {
    int zero = 0;
    SLtt_set_term_vtxxx(&zero);
+   Start_Alt_Chars_Str = "\016";
+   End_Alt_Chars_Str = "\017";
+   SLtt_Has_Alt_Charset = 1;
+   SLtt_Graphics_Char_Pairs = "aaffgghhjjkkllmmnnooqqssttuuvvwwxx";
+   Enable_Alt_Char_Set = "\033(B\033)0"; 
 }
 #endif
 
@@ -1389,3 +1495,22 @@ void SLtt_bold_video (void)
 {
    SLtt_write_string (Bold_Vid_Str);
 }
+
+int SLtt_set_mouse_mode (int mode, int force)
+{
+   char *term;
+   
+   if (force == 0)
+     {
+	if (NULL == (term = (char *) getenv("TERM"))) return -1;
+	if (strncmp ("xterm", term, 5))
+	  return -1;
+     }
+   
+   if (mode)
+     SLtt_write_string ("\033[?9h");
+   else
+     SLtt_write_string ("\033[?9l");
+   
+   return 0;
+}
