@@ -10,6 +10,9 @@
 
 #include <stdio.h>
 #include <string.h>
+
+#include "config.h"
+
 #include "slang.h"
 #include "_slang.h"
 
@@ -607,28 +610,24 @@ static void expression(void)
 	   case EOS_BRA: case COMMA:
 	     return;
 	     
-	   case COLON: 
-	     (*SLcompile_ptr)(":");
-	     CTok = EOS;
+	   case COLON:
 	     return;
+	     /* (*SLcompile_ptr)(":");
+	      * CTok = EOS;
+	      * return;
+	      */
 	     
 	   case NAME: 
 	     /* allow things like 'if (i == 2) i = 3;' and ':' */
 	     if ((*Token == ':') && (Token[1] == 0))
 	       {
 		  CTok = COLON;
-		  /* if_fudge = 1; */
 		  return;
 	       }
 	     
-	     /* if (if_fudge)
-	       {
-		  if_fudge = 0;
-		  return;
-	       } */
 	   default: 
 	     if (anything_terminates) return;
-	     parse_error("Expecting EOS", 0);
+	     parse_error("Expecting ';'", 0);
 	     return;
 	  }
      }
@@ -839,6 +838,8 @@ char *SLang_rpn_interpret(char *line)
    return(ret);
 }
 
+#if 0
+
 #define MAX_DEFINES 10
 #define MAX_DEFINE_LEN 16
 
@@ -999,13 +1000,14 @@ static int slang_line_ok (char *buf, int *levelp, int *exec_levelp)
    return (level == exec_level);
 }
    
+/* preprocessor */
 
+#endif  /* if 0 */
+
+static SLPreprocess_Type *This_SLpp;
 
 #define STREQS(a,b) (*(a) == *(b)) && !strcmp(a,b)
 
-/* preprocessor */
-static int SLpp_This_Level;
-static int SLpp_Exec_Level;
 
 
 #define MAX_PUSHED_TOKENS 3
@@ -1062,7 +1064,7 @@ static int get_token(void)
 		 || SLang_Error) return(CTok = END);
 	     
 	  }
-	while (!slang_line_ok(Input, &SLpp_This_Level, &SLpp_Exec_Level));
+	while (!SLprep_line_ok(Input, This_SLpp));
 	/* lines beginning with a '.' are RPN */
 	if (*Input == '.') return(CTok = INLINE_TOK);
 	else if ((*Input == '@') && (*at_handler_name)) return CTok = SPECIAL_AT_TOK;
@@ -1132,13 +1134,16 @@ static void SLang_eval_object(SLang_Load_Type *x)
 {
    char *(*last_read_fun)(SLang_Load_Type *) = Get_Token_Read_Fun;
    char *last_token = Token, *last_input = Input;
-   int last_level = SLpp_This_Level;
-   int last_exec_level = SLpp_Exec_Level;
+   SLPreprocess_Type *last_pp, this_pp;
    SLang_Load_Type *last_llt = LLT;   
    int last_ctok = CTok;
    
-   SLpp_This_Level = 0;
-   SLpp_Exec_Level = 0;
+   last_pp = This_SLpp;
+   
+   if (-1 == SLprep_open_prep (&this_pp))
+     return;
+   
+   This_SLpp = &this_pp;
 
    x->ofs = x->n = 0;
    Get_Token_Read_Fun = x->read;
@@ -1170,8 +1175,7 @@ static void SLang_eval_object(SLang_Load_Type *x)
    CTok = last_ctok;
    LLT = last_llt;
    Input = last_input;
-   SLpp_This_Level = last_level;
-   SLpp_Exec_Level = last_exec_level;
+   This_SLpp = last_pp;
 }
 
 
