@@ -138,26 +138,51 @@ static void SLdo_strcat(void)
    if (bdata == 1) SLFREE(b);
 }
 
+static char Utility_Char_Table [256];
+static void set_utility_char_table (char *pos)
+{
+   register char *t = Utility_Char_Table, *tmax;
+   register unsigned char ch;
+
+   tmax = t + 256;
+   while (t < tmax) *t++ = 0;
+   
+   t = Utility_Char_Table;
+   while ((ch = (unsigned char) *pos++) != 0) t[ch] = 1;
+}
+
+static int do_trim (char **beg, char **end, char *white)
+{
+   int len;
+   char *a, *b;
+   
+   set_utility_char_table (white);
+
+   a = *beg;
+   len = strlen (a);
+   b = a + (len - 1);
+   while (Utility_Char_Table[(unsigned char) *a]) a++;
+   while ((b >= a) && (Utility_Char_Table[(unsigned char) *b])) b--;
+   b++;
+   
+   len = (int) (b - a);
+   *beg = a;
+   *end = b;
+   return len;
+}
+
+
 static void SLdo_strtrim(void)
 {
-   char *a, *beg, *end, *c, *neew, ch;
+   char *a, *beg, *end, *c, *neew;
    int len;
    int adata;
 
    if (SLang_pop_string(&a, &adata)) return;
 
-   len = strlen(a);
    beg = a;
-   end = a + (len - 1);
-   while (ch = *beg, (ch == ' ') || (ch == '\t') || (ch == '\n')) beg++;
-   while (end >= beg)
-     {
-	ch = *end;
-	if ((ch == ' ') || (ch == '\t') || (ch == '\n')) end--;
-	else break;
-     }
-   end++;
-   len = (int) (end - beg);
+   len = do_trim (&beg, &end, " \t\n");
+   
    /* instead of going throug push string, push it directly.  This is because
     * the string that is being copied might be a constant. */
    if (NULL != (c = (char *) SLMALLOC(len + 1)))
@@ -173,7 +198,54 @@ static void SLdo_strtrim(void)
    if (adata == 1) SLFREE(a);
 }
 
-static char Utility_Char_Table [256];
+
+static void SLdo_strcompress (void)
+{
+   char *str, *white, *c;
+   unsigned char *s, *beg, *end;
+   int len;
+   int strdata, whitedata;
+
+   if (pop_two_strings (&str, &white, &strdata, &whitedata)) return;
+   
+   beg = (unsigned char *) str;
+   (void) do_trim ((char **) &beg, (char **) &end, white);
+   if (whitedata) SLFREE (white);
+
+   /* Determine the effective length */
+   len = 0;
+   s = (unsigned char *) beg;
+   while (s < end)
+     {
+	len++;
+	if (Utility_Char_Table[*s++])
+	  {
+	     while ((s < end) && Utility_Char_Table[*s]) s++;
+	  }
+     }
+   
+   if (NULL != (c = (char *) SLMALLOC(len + 1)))
+     {
+	s = (unsigned char *) c;
+	
+	while (beg < end)
+	  {
+	     *s++ = *beg;
+	     if (Utility_Char_Table[*beg++])
+	       {
+		  while ((beg < end) && Utility_Char_Table[*beg]) beg++;
+	       }
+	  }
+	
+	*s = 0;
+
+	SLang_push_malloced_string(c);
+     }
+   else SLang_Error = SL_MALLOC_ERROR; /* SLang_doerror("Lang Malloc error."); */
+   
+   if (strdata == 1) SLFREE(str);
+}
+
 
 static int str_replace (void)
 {
@@ -218,17 +290,6 @@ static int str_replace (void)
    return ret;
 }
 
-static void set_utility_char_table (char *pos)
-{
-   register char *t = Utility_Char_Table, *tmax;
-   register unsigned char ch;
-
-   tmax = t + 256;
-   while (t < tmax) *t++ = 0;
-   
-   t = Utility_Char_Table;
-   while ((ch = (unsigned char) *pos++) != 0) t[ch] = 1;
-}
 
 /* This routine returns the string with text removed between single character
    comment delimiters from the set b and e. */
@@ -287,6 +348,11 @@ static void SLquote_string (void)
    register unsigned char ch;
    
    if (SLang_pop_integer (&slash)) return;
+   if ((slash > 255) || (slash < 0))
+     {
+	SLang_Error = SL_INVALID_PARM;
+	return;
+     }
    
    if (pop_two_strings (&str, &quotes, &sdata, &qdata)) return;
    
@@ -340,28 +406,41 @@ static void SLdo_substr(void)
 {
    char *a;
    int adata, n, m;
-   char b[256];
+   char *b;
+   int lena;
 
    if (SLang_pop_integer(&m) || SLang_pop_integer(&n) || (SLang_pop_string(&a, &adata))) return;
-
-   *b = 0;
-   if (m > 0)
+   lena = strlen (a);
+   if (n > lena) n = lena + 1;
+   if (n < 1)
      {
-	strncpy(b, a + (n - 1), 254);
-	if (m > 255) m = 255;
-	b[m] = 0;
+	SLang_Error = SL_INVALID_PARM;
+     }
+   else
+     {
+	n--;
+	if (m < 0) m = 0; else if (n + m > lena) m = lena - n;
+	b = SLmake_nstring (a + n, m);
+	if (b != NULL)
+	  SLang_push_string(b);
      }
    if (adata == 1) SLFREE(a);
-   SLang_push_string(b);
 }
 /* substitute char m at positin string n in string*/
 static void SLdo_strsub(void)
 {
    char *a, *b;
    int adata, n, m;
-
+   int lena;
+   
    if (SLang_pop_integer(&m) || SLang_pop_integer(&n) || (SLang_pop_string(&a, &adata))) return;
-   if (NULL != (b = SLmake_string (a)))
+   lena = strlen (a);
+   
+   if ((lena < n) || (n <= 0))
+     {
+	SLang_Error = SL_INVALID_PARM;
+     }
+   else if (NULL != (b = SLmake_nstring (a, lena)))
      {
 	b[n - 1] = (char) m;
 	SLang_push_malloced_string (b);
@@ -425,8 +504,13 @@ static void SLdo_strncmp(void)
    
    if (SLang_pop_integer(&n) || pop_two_strings (&a, &b, &adata, &bdata))
      return;
-
-   SLang_push_integer (strncmp(a, b, n));
+   
+   if (n < 0)
+     {
+	SLang_Error = SL_INVALID_PARM;
+     }
+   else
+     SLang_push_integer (strncmp(a, b, n));
 
    if (adata) SLFREE(a);
    if (bdata) SLFREE(b);
@@ -500,12 +584,18 @@ static void SLdo_integer(void)
 {
    char *a;
    int adata;
-
+   int i;
+   
+   if (SLang_Error) return;
    if (SLang_pop_string(&a, &adata)) return;
-
-   /* Should check for parse error here but later */
-   SLang_push_integer (SLatoi((unsigned char *) a));
-
+   
+   i = SLatoi((unsigned char *) a);
+   if (SLang_Error)
+     {
+	SLang_Error = INTRINSIC_ERROR;
+	SLang_doerror ("The string cannot be converted to an integer.");
+     }
+   else SLang_push_integer (i);
    if (adata) SLFREE(a);
 }
 
@@ -595,7 +685,7 @@ static void lang_getenv_cmd(char *s)
    SLang_push_string(t);
 }
 
-#ifndef VMS
+#ifndef NO_PUTENV
 static void lang_putenv_cmd(void)
 {
    char *s;
@@ -1096,6 +1186,12 @@ static int my_system (char *s)
    return system (s);
 }
 
+/* These are here to let interpreter script have access to these values. */
+static int Integer_Type_Var = INT_TYPE;
+static int String_Type_Var = STRING_TYPE;
+#ifdef FLOAT_TYPE
+static int Float_Type_Var = FLOAT_TYPE;
+#endif
 
 SLang_Name_Type SLang_Basic_Table[] =
 {
@@ -1202,7 +1298,7 @@ SLang_Name_Type SLang_Basic_Table[] =
     * @     USE_ANSI_COLORS = 1;
     * @   }
     * Related Functions: @putenv@, @strlen@ */
-#ifndef VMS
+#ifndef NO_PUTENV
    MAKE_INTRINSIC(".putenv",  lang_putenv_cmd, VOID_TYPE, 0),
    /* Prototype: Void putenv (String s);
     * This functions adds string @s@ to the environment.  Typically, @s@ should
@@ -1525,8 +1621,20 @@ SLang_Name_Type SLang_Basic_Table[] =
     * The @strtrim@ function removes all leading and trailing whitespace
     * characters from the string @s@ and returns the result.  Whitespace is
     * defined to be any combination of spaces, tabs, and newline characters.
-    * Related Functions: @int@, @strlow@, @strup@
+    * Related Functions: @int@, @strlow@, @strup@, @strcompress@
     */
+     
+   MAKE_INTRINSIC(".strcompress", SLdo_strcompress, VOID_TYPE, 0),
+   /* Prototype: String strtrim (String s, String white);
+    * The @strcompress@ function compresses the string @s@ by removing all 
+    * repeated characters specified by @white@ from the interior of @s@.  In
+    * addition, it also removes all leading and trailing characters from @s@
+    * that are part of @white@.  For example,
+    * @ strcompress (",;apple,,cherry;,banana", ",;");
+    * returns the string @"apple,cherry;banana"@.
+    * Related Functions: @strtrim@
+    */
+     
 
    MAKE_INTRINSIC(".int",  SLdo_int, VOID_TYPE, 0),
    /* Prototype: Integer int (String s);
@@ -1663,7 +1771,6 @@ SLang_Name_Type SLang_Basic_Table[] =
     * Related Variables: @_slangtrace@, @_traceback@
     */
 	
-   MAKE_VARIABLE("._slang_version", &SLang_Version, INT_TYPE, 1),
    MAKE_INTRINSIC(".set_float_format", set_float_format, VOID_TYPE, 1),
    /* Prototype: Void set_float_format (String fmt);
     * This function is used to set the floating point format to be used
@@ -1679,6 +1786,41 @@ SLang_Name_Type SLang_Basic_Table[] =
     * Related Functions: @create_array@, @dup@
     */
    
+   MAKE_INTRINSIC(".array_info", SLarray_info, VOID_TYPE, 0),
+   /* Prototype: Var array_info (Array a);
+    * This function returns information about the array @a@.  The number of
+    * values returned depend on the type of array.  It always returns at
+    * least three values to the stack.  For two and three dimensional
+    * arrays, it returns four and fives values, resp.
+    * The values are ordered in such a way that it may be used as an argument 
+    * to the @create_array@ function.  For example, 
+    * @ define duplicate_array (a)
+    * @ {
+    * @   variable b = create_array (array_info (a));
+    * @   return copy_array (b, a);
+    * @ }
+    * See also: @create_array@, @copy_array@
+    */
+   MAKE_INTRINSIC("._slang_guess_type", SLang_guess_type, INT_TYPE, 1),
+   /* Prototype: Integer _slang_guess_type (String s);
+    * This function tries to determine whether its argument @s@ represents 
+    * an integer or a floating point number.  If it appears to be neither, 
+    * then a string is assumed.  It returns one of three values depending on
+    * the format of the string @s@:
+    * @ INT_TYPE     :   If it appears to be an integer
+    * @ FLOAT_TYPE   :   If it appears to be a float
+    * @ STRING_TYPE  :   Anything else.
+    * For example, @_slang_guess_type("1e2")@ returns @FLOAT_TYPE@ but 
+    * @_slang_guess_type("e12")@ returns @STRING_TYPE@.
+    * Related Functions: @integer@, @string@, @float@
+    * Related Variables: @INT_TYPE@, @STRING_TYPE@, @FLOAT_TYPE@
+    */
+   MAKE_VARIABLE("._slang_version", &SLang_Version, INT_TYPE, 1),
+   MAKE_VARIABLE(".INT_TYPE", &Integer_Type_Var, INT_TYPE, 1),
+   MAKE_VARIABLE(".STRING_TYPE", &String_Type_Var, INT_TYPE, 1),
+#ifdef FLOAT_TYPE
+   MAKE_VARIABLE(".FLOAT_TYPE", &Float_Type_Var, INT_TYPE, 1),
+#endif
    SLANG_END_TABLE
 };
 
@@ -1861,13 +2003,24 @@ int init_SLang()
 # endif
 #endif
 
+#ifdef MSWINDOWS
+   SLang_Name_Table = (SLang_Name_Type *) SLCALLOC(SLANG_MAX_SYMBOLS, sizeof (SLang_Name_Type));
+   if (SLang_Name_Table == NULL) return 0;
+#endif
+   
    if (0 == register_types ()) return 0;
 
    if (!SLang_add_table(SLang_Basic_Table, "_Basic")) return(0);
    SLadd_variable(SLANG_SYSTEM_NAME);
    
    if (!SLdefine_for_ifdef (s)) return 0;
-   
+#ifdef MSWINDOWS
+   /* I am not sure whether this belongs here or not. */
+   if (!SLdefine_for_ifdef ("MSWINDOWS")) return 0;
+#endif
+#ifdef FLOAT_TYPE
+   if (!SLdefine_for_ifdef ("FLOAT_TYPE")) return 0;
+#endif
    /* give temp global variables $0 --> $9 */
    name[2] = 0; name[0] = '$';
    for (i = 0; i < 10; i++)
